@@ -31,7 +31,7 @@ CartridgeMDMWidget::CartridgeMDMWidget(
 
   ostringstream info;
   info << "Menu Driven Megacart, containing up to 128 4K banks\n"
-       << "Startup bank = " << cart.myStartBank << "\n"
+       << "Startup bank = " << cart.startBank() << "\n"
        << "\nBanks are selected by reading from $800 - $BFF, where the lower "
           "byte determines the 4K bank to use.";
 
