@@ -30,7 +30,7 @@ CartridgeUAWidget::CartridgeUAWidget(
 
   ostringstream info;
   info << "8K UA cartridge, two 4K banks\n"
-       << "Startup bank = " << cart.myStartBank << " or undetermined\n";
+       << "Startup bank = " << cart.startBank() << " or undetermined\n";
 
   // Eventually, we should query this from the debugger/disassembler
   for(uInt32 i = 0, offset = 0xFFC, spot = 0x220; i < 2;
