@@ -20,8 +20,8 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Cartridge4KSC::Cartridge4KSC(const BytePtr& image, uInt32 size,
-                             const Settings& settings)
-  : Cartridge(settings)
+                             const string& md5, const Settings& settings)
+  : Cartridge(settings, md5)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image.get(), std::min(4096u, size));
@@ -44,16 +44,16 @@ void Cartridge4KSC::install(System& system)
   System::PageAccess access(this, System::PA_READ);
 
   // Set the page accessing method for the RAM writing pages
+  // Map access to this class, since we need to inspect all accesses to
+  // check if RWP happens
   access.type = System::PA_WRITE;
   for(uInt16 addr = 0x1000; addr < 0x1080; addr += System::PAGE_SIZE)
   {
-    access.directPokeBase = &myRAM[addr & 0x007F];
     access.codeAccessBase = &myCodeAccessBase[addr & 0x007F];
     mySystem->setPageAccess(addr, access);
   }
 
   // Set the page accessing method for the RAM reading pages
-  access.directPokeBase = nullptr;
   access.type = System::PA_READ;
   for(uInt16 addr = 0x1080; addr < 0x1100; addr += System::PAGE_SIZE)
   {
@@ -75,17 +75,16 @@ void Cartridge4KSC::install(System& system)
 uInt8 Cartridge4KSC::peek(uInt16 address)
 {
   // The only way we can get to this method is if we attempt to read from
-  // the write port (0xF000 - 0xF080, 128 bytes), in which case an
-  // unwanted write is triggered
-  uInt8 value = mySystem->getDataBusState(0xFF);
+  // the write port (0xF000 - 0xF07F, 128 bytes), in which case an
+  // unwanted write is potentially triggered
+  return peekRAM(myRAM[address & 0x007F], address);
+}
 
-  if(bankLocked())
-    return value;
-  else
-  {
-    triggerReadFromWritePort(address);
-    return myRAM[address & 0x0FFF] = value;
-  }
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+bool Cartridge4KSC::poke(uInt16 address, uInt8 value)
+{
+  pokeRAM(myRAM[address & 0x007F], address, value);
+  return true;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -118,7 +117,6 @@ bool Cartridge4KSC::save(Serializer& out) const
 {
   try
   {
-    out.putString(name());
     out.putByteArray(myRAM, 128);
   }
   catch(...)
@@ -135,9 +133,6 @@ bool Cartridge4KSC::load(Serializer& in)
 {
   try
   {
-    if(in.getString() != name())
-      return false;
-
     in.getByteArray(myRAM, 128);
   }
   catch(...)
