@@ -22,17 +22,14 @@
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeCM::CartridgeCM(const BytePtr& image, uInt32 size,
-                         const Settings& settings)
-  : Cartridge(settings),
+                         const string& md5, const Settings& settings)
+  : Cartridge(settings, md5),
     mySWCHA(0xFF),   // portA is all 1's
     myBankOffset(0)
 {
   // Copy the ROM image into my buffer
   memcpy(myImage, image.get(), std::min(16384u, size));
   createCodeAccessBase(16384);
-
-  // On powerup, the last bank of ROM is enabled and RAM is disabled
-  myStartBank = mySWCHA & 0x3;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -40,8 +37,12 @@ void CartridgeCM::reset()
 {
   initializeRAM(myRAM, 2048);
 
+  // On powerup, the last bank of ROM is enabled and RAM is disabled
+  mySWCHA = 0xFF;
+  initializeStartBank(mySWCHA & 0x3);
+
   // Upon reset we switch to the startup bank
-  bank(myStartBank);
+  bank(startBank());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -54,7 +55,7 @@ void CartridgeCM::install(System& system)
   mySystem->m6532().installDelegate(system, *this);
 
   // Install pages for the startup bank
-  bank(myStartBank);
+  bank(startBank());
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -190,7 +191,6 @@ bool CartridgeCM::save(Serializer& out) const
 {
   try
   {
-    out.putString(name());
     out.putShort(myBankOffset);
     out.putByte(mySWCHA);
     out.putByte(myCompuMate->column());
@@ -210,9 +210,6 @@ bool CartridgeCM::load(Serializer& in)
 {
   try
   {
-    if(in.getString() != name())
-      return false;
-
     myBankOffset = in.getShort();
     mySWCHA = in.getByte();
     myCompuMate->column() = in.getByte();
