@@ -21,8 +21,8 @@
 
 //  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 CartridgeDASH::CartridgeDASH(const BytePtr& image, uInt32 size,
-                             const Settings& settings)
-  : Cartridge(settings),
+                             const string& md5, const Settings& settings)
+  : Cartridge(settings, md5),
     mySize(size)
 {
   // Allocate array for the ROM image
@@ -31,10 +31,6 @@ CartridgeDASH::CartridgeDASH(const BytePtr& image, uInt32 size,
   // Copy the ROM image into my buffer
   memcpy(myImage.get(), image.get(), mySize);
   createCodeAccessBase(mySize + RAM_TOTAL_SIZE);
-
-  // Remember startup bank (0 per spec, rather than last per 3E scheme).
-  // Set this to go to 3rd 1K Bank.
-  myStartBank = 0;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -42,9 +38,13 @@ void CartridgeDASH::reset()
 {
   initializeRAM(myRAM, RAM_TOTAL_SIZE);
 
+  // Remember startup bank (0 per spec, rather than last per 3E scheme).
+  // Set this to go to 3rd 1K Bank.
+  initializeStartBank(0);
+
   // Initialise bank values for all ROM/RAM access
   // This is used to reverse-lookup from address to bank location
-  for(uInt32 b = 0; b < 8; b++)
+  for(uInt32 b = 0; b < 8; ++b)
   {
     bankInUse[b] = BANK_UNDEFINED;        // bank is undefined and inaccessible!
     segmentInUse[b/2] = BANK_UNDEFINED;
@@ -69,7 +69,7 @@ void CartridgeDASH::install(System& system)
 
   // Initialise bank values for all ROM/RAM access
   // This is used to reverse-lookup from address to bank location
-  for (uInt32 b = 0; b < 8; b++)
+  for (uInt32 b = 0; b < 8; ++b)
   {
     bankInUse[b] = BANK_UNDEFINED;        // bank is undefined and inaccessible!
     segmentInUse[b/2] = BANK_UNDEFINED;
@@ -92,28 +92,19 @@ uInt8 CartridgeDASH::peek(uInt16 address)
   uInt32 bank = (address >> (ROM_BANK_TO_POWER - 1)) & 7;   // convert to 512 byte bank index (0-7)
   uInt16 imageBank = bankInUse[bank];                       // the ROM/RAM bank that's here
 
-  if (imageBank == BANK_UNDEFINED)            // an uninitialised bank?
+  if(imageBank == BANK_UNDEFINED)            // an uninitialised bank?
   {
     // accessing invalid bank, so return should be... random?
     value = mySystem->randGenerator().next();
 
   }
-  else if (imageBank & BITMASK_ROMRAM)        // a RAM bank
+  else if(imageBank & BITMASK_ROMRAM)        // a RAM bank
   {
-    // Reading from the write port triggers an unwanted write
-    value = mySystem->getDataBusState(0xFF);
+    Int32 ramBank = imageBank & BIT_BANK_MASK;    // discard irrelevant bits
+    Int32 offset = ramBank << RAM_BANK_TO_POWER;  // base bank address in RAM
+    offset += (address & BITMASK_RAM_BANK);       // + byte offset in RAM bank
 
-    if(bankLocked())
-      return value;
-    else
-    {
-      triggerReadFromWritePort(peekAddress);
-
-      Int32 ramBank = imageBank & BIT_BANK_MASK;    // discard irrelevant bits
-      Int32 offset = ramBank << RAM_BANK_TO_POWER;  // base bank address in RAM
-      offset += (address & BITMASK_RAM_BANK);       // + byte offset in RAM bank
-      return myRAM[offset] = value;
-    }
+    return peekRAM(myRAM[offset], peekAddress);
   }
 
   return value;
@@ -129,12 +120,27 @@ bool CartridgeDASH::poke(uInt16 address, uInt8 value)
 
   if (address == BANK_SWITCH_HOTSPOT_RAM)
     changed = bankRAM(value);
-
   else if (address == BANK_SWITCH_HOTSPOT_ROM)
     changed = bankROM(value);
 
-  // Handle TIA space that we claimed above
-  mySystem->tia().poke(address, value);
+  if(!(address & 0x1000))
+  {
+    // Handle TIA space that we claimed above
+    changed = changed || mySystem->tia().poke(address, value);
+  }
+  else
+  {
+    uInt32 bankNumber = (address >> RAM_BANK_TO_POWER) & 7;   // now 512 byte bank # (ie: 0-7)
+    Int16 whichBankIsThere = bankInUse[bankNumber];           // ROM or RAM bank reference
+
+    if(whichBankIsThere & BITMASK_ROMRAM)
+    {
+      uInt32 byteOffset = address & BITMASK_RAM_BANK;
+      uInt32 baseAddress = ((whichBankIsThere & BIT_BANK_MASK) << RAM_BANK_TO_POWER) + byteOffset;
+      pokeRAM(myRAM[baseAddress], address, value);
+      changed = true;
+    }
+  }
 
   return changed;
 }
@@ -183,9 +189,7 @@ void CartridgeDASH::bankRAMSlot(uInt16 bank)
 
   for (uInt16 addr = start; addr <= end; addr += System::PAGE_SIZE)
   {
-    if(upper)
-      access.directPokeBase = &myRAM[startCurrentBank + (addr & (RAM_BANK_SIZE - 1))];
-    else
+    if(!upper)
       access.directPeekBase = &myRAM[startCurrentBank + (addr & (RAM_BANK_SIZE - 1))];
 
     access.codeAccessBase = &myCodeAccessBase[mySize + startCurrentBank + (addr & (RAM_BANK_SIZE - 1))];
@@ -240,7 +244,7 @@ void CartridgeDASH::bankROMSlot(uInt16 bank)
 void CartridgeDASH::initializeBankState()
 {
   // Switch in each 512b slot
-  for(uInt32 b = 0; b < 8; b++)
+  for(uInt32 b = 0; b < 8; ++b)
   {
     if(bankInUse[b] == BANK_UNDEFINED)
     {
@@ -307,7 +311,6 @@ bool CartridgeDASH::save(Serializer& out) const
 {
   try
   {
-    out.putString(name());
     out.putShortArray(bankInUse, 8);
     out.putShortArray(segmentInUse, 4);
     out.putByteArray(myRAM, RAM_TOTAL_SIZE);
@@ -325,8 +328,6 @@ bool CartridgeDASH::load(Serializer& in)
 {
   try
   {
-    if (in.getString() != name())
-      return false;
     in.getShortArray(bankInUse, 8);
     in.getShortArray(segmentInUse, 4);
     in.getByteArray(myRAM, RAM_TOTAL_SIZE);
