@@ -23,19 +23,18 @@
 CompuMate::CompuMate(const Console& console, const Event& event,
                      const System& system)
   : myConsole(console),
-    myEvent(event),
     myColumn(0),
-    myKeyTable(nullptr)
+    myKeyTable(event.getKeys())
 {
   // These controller pointers will be retrieved by the Console, which will
   // also take ownership of them
   myLeftController  = make_unique<CMControl>(*this, Controller::Left, event, system);
   myRightController = make_unique<CMControl>(*this, Controller::Right, event, system);
 
-  myLeftController->updateAnalogPin(Controller::Nine, Controller::maximumResistance);
-  myLeftController->updateAnalogPin(Controller::Five, Controller::minimumResistance);
-  myRightController->updateAnalogPin(Controller::Nine, Controller::minimumResistance);
-  myRightController->updateAnalogPin(Controller::Five, Controller::maximumResistance);
+  myLeftController->updateAnalogPin(Controller::Nine, Controller::MAX_RESISTANCE);
+  myLeftController->updateAnalogPin(Controller::Five, Controller::MIN_RESISTANCE);
+  myRightController->updateAnalogPin(Controller::Nine, Controller::MIN_RESISTANCE);
+  myRightController->updateAnalogPin(Controller::Five, Controller::MAX_RESISTANCE);
 
   enableKeyHandling(false);
 }
@@ -43,15 +42,7 @@ CompuMate::CompuMate(const Console& console, const Event& event,
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void CompuMate::enableKeyHandling(bool enable)
 {
-  if(enable)
-    myKeyTable = myEvent.getKeys();
-  else
-  {
-    for(uInt32 i = 0; i < KBDK_LAST; ++i)
-      myInternalKeyTable[i] = false;
-
-    myKeyTable = myInternalKeyTable;
-  }
+  myKeyTable.enable(enable);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -61,17 +52,17 @@ void CompuMate::update()
   Controller& lp = myConsole.leftController();
   Controller& rp = myConsole.rightController();
 
-  lp.myAnalogPinValue[Controller::Nine] = Controller::maximumResistance;
-  lp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+  lp.myAnalogPinValue[Controller::Nine] = Controller::MAX_RESISTANCE;
+  lp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
   lp.myDigitalPinState[Controller::Six] = true;
-  rp.myAnalogPinValue[Controller::Nine] = Controller::minimumResistance;
-  rp.myAnalogPinValue[Controller::Five] = Controller::maximumResistance;
+  rp.myAnalogPinValue[Controller::Nine] = Controller::MIN_RESISTANCE;
+  rp.myAnalogPinValue[Controller::Five] = Controller::MAX_RESISTANCE;
   rp.myDigitalPinState[Controller::Six] = true;
 
   if (myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT])
-    rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+    rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
   if (myKeyTable[KBDK_LCTRL] || myKeyTable[KBDK_RCTRL])
-    lp.myAnalogPinValue[Controller::Nine] = Controller::minimumResistance;
+    lp.myAnalogPinValue[Controller::Nine] = Controller::MIN_RESISTANCE;
 
   rp.myDigitalPinState[Controller::Three] = true;
   rp.myDigitalPinState[Controller::Four] = true;
@@ -89,7 +80,7 @@ void CompuMate::update()
       // Emulate the '?' character (Shift-6) with the actual question key
       if (myKeyTable[KBDK_SLASH] && (myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_Y]) rp.myDigitalPinState[Controller::Three] = false;
@@ -101,7 +92,7 @@ void CompuMate::update()
       // Emulate the '[' character (Shift-8) with the actual key
       if (myKeyTable[KBDK_LEFTBRACKET] && !(myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_I]) rp.myDigitalPinState[Controller::Three] = false;
@@ -113,7 +104,7 @@ void CompuMate::update()
       // Emulate the '-' character (Shift-2) with the actual minus key
       if (myKeyTable[KBDK_MINUS] && !(myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_W]) rp.myDigitalPinState[Controller::Three] = false;
@@ -131,7 +122,7 @@ void CompuMate::update()
       // Emulate the quote character (Shift-0) with the actual quote key
       if (myKeyTable[KBDK_APOSTROPHE] && (myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_P]) rp.myDigitalPinState[Controller::Three] = false;
@@ -141,7 +132,7 @@ void CompuMate::update()
       // Emulate Ctrl-space (aka backspace) with the actual Backspace key
       if (myKeyTable[KBDK_BACKSPACE])
       {
-        lp.myAnalogPinValue[Controller::Nine] = Controller::minimumResistance;
+        lp.myAnalogPinValue[Controller::Nine] = Controller::MIN_RESISTANCE;
         rp.myDigitalPinState[Controller::Four] = false;
       }
       break;
@@ -150,7 +141,7 @@ void CompuMate::update()
       // Emulate the ']' character (Shift-9) with the actual key
       if (myKeyTable[KBDK_RIGHTBRACKET] && !(myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_O]) rp.myDigitalPinState[Controller::Three] = false;
@@ -162,7 +153,7 @@ void CompuMate::update()
       // Emulate the '=' character (Shift-5) with the actual equals key
       if (myKeyTable[KBDK_EQUALS] && !(myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_T]) rp.myDigitalPinState[Controller::Three] = false;
@@ -174,7 +165,7 @@ void CompuMate::update()
       // Emulate the '+' character (Shift-1) with the actual plus key (Shift-=)
       if (myKeyTable[KBDK_EQUALS] && (myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_Q]) rp.myDigitalPinState[Controller::Three] = false;
@@ -186,7 +177,7 @@ void CompuMate::update()
       // Emulate the '/' character (Shift-4) with the actual slash key
       if (myKeyTable[KBDK_SLASH] && !(myKeyTable[KBDK_LSHIFT] || myKeyTable[KBDK_RSHIFT]))
       {
-        rp.myAnalogPinValue[Controller::Five] = Controller::minimumResistance;
+        rp.myAnalogPinValue[Controller::Five] = Controller::MIN_RESISTANCE;
         lp.myDigitalPinState[Controller::Six] = false;
       }
       if (myKeyTable[KBDK_R]) rp.myDigitalPinState[Controller::Three] = false;
