@@ -39,6 +39,7 @@ EditableWidget::EditableWidget(GuiObject* boss, const GUI::Font& font,
 
   _bgcolor = kWidColor;
   _bgcolorhi = kWidColor;
+  _bgcolorlo = kDlgColor;
   _textcolor = kTextColor;
   _textcolorhi = kTextColor;
 
@@ -453,13 +454,13 @@ bool EditableWidget::moveWord(int direction)
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditableWidget::copySelectedText()
 {
-  _clippedText = _editString;
+//  _clippedText = _editString;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 void EditableWidget::pasteSelectedText()
 {
-  _editString = _clippedText;
+//  _editString = _clippedText;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
